// Header file for USB+Ethernet/DMX Generator  ********************************
//
// Simple DMX512(incl. Art-Net) controller for STM32 Nucleo-L476 + PCB
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
//
// supports the essential features of Art-Net, the MiniDMX protocol
// plus simple MIDI message and ASCII text based command sets
//
// Author and (c)2017 Wolfgang Schemmert. The copyright claim and GPL-3 Licence
// is restricted to this file and the depending files
// "main.c", "usb-otg.c", "dmx.c", "ether.c" and "userinterface.c".
// Other depending material is published and copyrighted
// by STMicroelectronics, Atollic AB and EMBITZ.org
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction and operation manual is provided at this website
//
// Note: at present state of art, only the DEBUG object code is working.
// The RELEASE version compiled with EmBitz1.11 does not work. Reason unkown.
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// **************************************************************************


// extern defined in userinterface.c  ************
extern uint8_t gcPushByte, gcIgnoreVelo0;
extern void  AsciiCmd_Interpreter(uint8_t);
extern void  MidiCmd_Interpreter(uint32_t);
extern void  Setup_Interpreter(uint8_t);

// extern defined in usb-fs.c  ************
extern uint8_t guDeviceClass, guConfigured;
extern uint8_t guUsbCmdEnable;      //simply to speed up sendCmdB() a little bit
extern void stopUSB(void);
extern void initUSB(void);
extern uint16_t pullUsbRxBuf(void);
extern void sendUsbByte(uint8_t);
extern uint32_t pullMidiMsg(void);

// extern defined in ether.c  ************
extern uint8_t geSPI_RXBUF[];
extern uint8_t *geSPI_RXHEAD, *geSPI_RXTAIL, *geSPI_RXBUFTOP;
extern uint8_t geSPI_TXBUF[];
extern uint8_t *geSPI_TXHEAD, *geSPI_TXTAIL, *geSPI_TXBUFTOP;
extern uint8_t geArtnetOn;  //1=default: Artnet data are send to DMX. 0=block

extern void initSPI1(void);
uint8_t exchangeSpiByte(uint8_t);
extern uint8_t initW5500(void);
extern uint8_t openUDP0(void);
extern uint8_t sendEtherByte(uint8_t);
extern uint8_t receiveUDP(void);
extern void writeWiz_4(uint16_t, uint8_t, uint8_t);
extern uint8_t readWiz_4(uint16_t, uint8_t);
extern void writeWiz_5(uint16_t, uint8_t, uint16_t);
extern uint16_t readWiz_5(uint16_t, uint8_t);
extern void writeWiz_7(uint16_t, uint8_t, uint32_t);
extern void writeWiz_Many(uint16_t, uint8_t, uint8_t *, uint8_t);
extern void readWiz_Data(uint16_t, uint8_t);

// extern defined in dmx.c
extern uint8_t gdDMXBUF[DMX_BUFLEN+7], gdFINALMEM[DMX_BUFLEN+7];
extern uint8_t gdFaderBusy, gdMiniDmxOn; //gdDmxCycle,
extern uint16_t gdDmxTxIndex, gdFadeTime, gdDmxMax;
extern void startDmxTx(void);
extern void DmxHeader(void);
extern void MiniDMX_Interpreter(void);
extern void fade2Preset(void);
extern void fadeProcess(void);
extern void sendAddrLevel(void);
extern void sendLevel(uint16_t);

// ****************************************

//system control
uint16_t gSYSCOUNT, gGREEN_LEDCOUNT, gRED_LEDCOUNT, gDELAYCOUNT, gTIMEOUT;
uint8_t gOpMode;            //0=Setup, 1=UDP/COM, 2=TCP/COM, 0x1x=MIDI Merge
uint16_t gSwitches;
volatile uint32_t gTest;    //used to check system variables without SVD

#define PAGE15 (0x08007800)
#define PAGE16 (0x08008000)

struct PARAM{
uint32_t VidPid;
uint32_t MacHi;
uint32_t MacLo;
uint32_t LocalIP;
uint32_t Subnet;
uint32_t Gateway;
uint32_t ArtnetIP;
uint32_t ArtSubnet;
uint32_t ArtUniverse;
uint32_t Channel;
uint32_t Reserved;
} gComParam;  //10 words

uint16_t gVidShadow, gPidShadow;

uint8_t gNoUsbStr[] = {" noUSB! "};

void SetSysClock(void);
void msDelay(uint16_t);
void wait1us(void);

void popFlashPage15(void);
void pushFlashPage15(void);
void popPresetPage(uint32_t);
void pushPresetPage(uint32_t);
void servSystem(void);

//USART
uint8_t gUSART_RXBUF[USART_BUFLEN];
uint8_t *gUSART_RXHEAD=0,*gUSART_RXTAIL=0,*gUSART_RXBUFTOP= &(gUSART_RXBUF[USART_BUFLEN]);

void initUsart2(void);
void sendB2(uint8_t);
uint16_t pullUsartRxBuf(void);
void sendCmdB(uint8_t);
void sendX8(uint8_t);
void sendN8(uint8_t);
void sendNibble(uint8_t);
void sendString(const uint8_t *);
void sendAllNibble(uint8_t);
void sendX16(uint16_t);
void sendX32(uint32_t);
void sendN32(uint32_t);         //S32 without trailing SPACE
void sendS32(uint32_t);
uint16_t pullCmdRxBuf(void);

#ifdef DEBUG
uint8_t gDBGBUF[USART_BUFLEN*8];  //4096 bytes
uint8_t *gDBGHEAD=gDBGBUF,*gDBGBUFTOP=&(gDBGBUF[USART_BUFLEN*8]);

void initSDBG(void);
void sdbgB(uint8_t);
void sdbgX8(uint8_t);
void sdbgNibble(uint8_t);
void sdbgX16(uint16_t);
void sdbgX32(uint32_t);
void sendDebug(void);
#endif

//Interrupt handlers
void SysTick_Handler(void);
void USART2_IRQHandler(void);
